diff --git a/plugins/global/nvidia/nvidia.cpp b/plugins/global/nvidia/nvidia.cpp
index 26eb790d..5133d071 100644
--- a/plugins/global/nvidia/nvidia.cpp
+++ b/plugins/global/nvidia/nvidia.cpp
@@ -78,6 +78,7 @@ public slots:
     };
 
 private:
+    int index;
     QTimer* timer;
     QProcess *m_process_alloc;
 
@@ -95,6 +96,7 @@ private:
 GPU::GPU(int index, SensorContainer *parent)
     : SensorObject(QStringLiteral("gpu%1").arg(index), i18nc("@title", "GPU %1", index + 1), parent)
 {
+    this->index= index;
     auto displayIndex = index + 1;
 
     timer= new QTimer();
@@ -157,6 +159,34 @@ GPU::GPU(int index, SensorContainer *parent)
     m_processorClock->setVariantType(QVariant::UInt);
 }
 
+int NvidiaPlugin::getGpuNum() {
+    int num = 0;
+
+    auto const sniExecutable = QStandardPaths::findExecutable("nvidia-smi");
+    if (sniExecutable.isEmpty()) {
+        return num;
+    }
+
+    auto process= new QProcess(this);
+    process->setProgram(sniExecutable);
+    process->setArguments({QStringList()
+        << "-L"});
+
+    connect(process, &QProcess::readyReadStandardOutput, this, [=, &num]() {
+        while (process->canReadLine()) {
+            QString const line= process->readLine();
+            num++;
+        }
+        process->terminate();
+        process->deleteLater();
+    });
+
+    process->start();
+    process->waitForFinished();
+
+    return num;
+}
+
 NvidiaPlugin::NvidiaPlugin(QObject *parent, const QVariantList &args)
     : SensorPlugin(parent, args)
 {
@@ -167,10 +197,11 @@ NvidiaPlugin::NvidiaPlugin(QObject *parent, const QVariantList &args)
 
     auto gpuSystem = new SensorContainer("nvidia", i18nc("@title NVidia GPU information", "NVidia"), this);
 
-    //assuming just one GPU for now
-    auto gpu0 = new GPU(0, gpuSystem);
-    connect(gpu0, &SensorObject::subscribedChanged, this, &NvidiaPlugin::gpuSubscriptionChanged);
-    m_gpus[0] = gpu0;
+    for (int i = 0; i < getGpuNum(); i++) {
+        auto gpu = new GPU(i, gpuSystem);
+        connect(gpu, &SensorObject::subscribedChanged, this, &NvidiaPlugin::gpuSubscriptionChanged);
+        m_gpus[i] = gpu;
+    }
 
     m_process = new QProcess(this);
     m_process->setProgram(sniExecutable);
@@ -217,7 +248,7 @@ void NvidiaPlugin::gpuSubscriptionChanged(bool subscribed)
 {
     if (subscribed) {
         m_activeWatcherCount++;
-        if (m_activeWatcherCount == 1) {
+        if (m_activeWatcherCount >= 1) {
             m_process->start();
         }
     } else {
diff --git a/plugins/global/nvidia/nvidia.h b/plugins/global/nvidia/nvidia.h
index 82d8e42a..3aea81e7 100644
--- a/plugins/global/nvidia/nvidia.h
+++ b/plugins/global/nvidia/nvidia.h
@@ -40,6 +40,7 @@ public:
 
 private:
     void gpuSubscriptionChanged(bool subscribed);
+    int getGpuNum();
 
     QProcess *m_process;
     QHash<int, GPU *> m_gpus;
